@@ -5,12 +5,14 @@ extern crate cc;
 extern crate libc;
 #[cfg(target_env = "msvc")]
 extern crate vcpkg;
+#[cfg(target_env = "msvc")]
+extern crate zip;
 
 extern crate libflate;
 extern crate minisign_verify;
 extern crate pkg_config;
 extern crate tar;
-#[cfg(feature = "fetch-latest")]
+#[cfg(any(windows, feature = "fetch-latest"))]
 extern crate ureq;
 
 use std::{
@@ -128,11 +130,46 @@ fn find_libsodium_pkg() {
     }
 }
 
-#[cfg(windows)]
-fn make_libsodium(_: &str, _: &Path, _: &Path) -> PathBuf {
-    // We don't build anything on windows, we simply linked to precompiled
-    // libs.
-    get_lib_dir()
+#[cfg(all(windows, target_env = "msvc"))]
+fn make_libsodium(_: &str, _: &Path, install_dir: &Path) -> PathBuf {
+    // We don't build anything on windows, we simply link to precompiled libs.
+    use zip::read::ZipArchive;
+
+    // Determine filename for pre-built MSVC binaries
+    let basename = "libsodium-1.0.18-stable-msvc";
+    let filename = format!("{}.zip", basename);
+    let signature_filename = format!("{}.zip.minisig", basename);
+
+    // Read binaries archive from disk (or download if requested) & verify signature
+    let archive_bin = retrieve_and_verify_archive(&filename, &signature_filename);
+
+    // Unpack the zip
+    let mut archive = ZipArchive::new(std::io::Cursor::new(archive_bin)).unwrap();
+    archive.extract(&install_dir).unwrap();
+
+    get_lib_dir(install_dir)
+}
+
+#[cfg(all(windows, not(target_env = "msvc")))]
+fn make_libsodium(_: &str, _: &Path, install_dir: &Path) -> PathBuf {
+    // We don't build anything on windows, we simply link to precompiled libs.
+    use libflate::gzip::Decoder;
+    use tar::Archive;
+
+    // Determine filename for pre-built MinGW binaries
+    let basename = "libsodium-1.0.18-stable-mingw";
+    let filename = format!("{}.tar.gz", basename);
+    let signature_filename = format!("{}.tar.gz.minisig", basename);
+
+    // Read binaries archive from disk (or download if requested) & verify signature
+    let archive_bin = retrieve_and_verify_archive(&filename, &signature_filename);
+
+    // Unpack the tarball
+    let gz_decoder = Decoder::new(std::io::Cursor::new(archive_bin)).unwrap();
+    let mut archive = Archive::new(gz_decoder);
+    archive.unpack(&install_dir).unwrap();
+
+    get_lib_dir(install_dir)
 }
 
 #[cfg(not(windows))]
@@ -343,80 +380,48 @@ fn is_release_profile() -> bool {
 }
 
 #[cfg(all(target_env = "msvc", target_pointer_width = "32"))]
-fn get_lib_dir() -> PathBuf {
+fn get_lib_dir(install_dir: &Path) -> PathBuf {
     if is_release_profile() {
-        get_crate_dir().join("msvc/Win32/Release/v140/")
+        install_dir.join("libsodium/Win32/Release/v143/static/")
     } else {
-        get_crate_dir().join("msvc/Win32/Debug/v140/")
+        install_dir.join("libsodium/Win32/Debug/v143/static/")
     }
 }
 
 #[cfg(all(target_env = "msvc", target_pointer_width = "64"))]
-fn get_lib_dir() -> PathBuf {
+fn get_lib_dir(install_dir: &Path) -> PathBuf {
     if is_release_profile() {
-        get_crate_dir().join("msvc/x64/Release/v140/")
+        install_dir.join("libsodium/x64/Release/v143/static/")
     } else {
-        get_crate_dir().join("msvc/x64/Debug/v140/")
+        install_dir.join("libsodium/x64/Debug/v143/static/")
     }
 }
 
 #[cfg(all(windows, not(target_env = "msvc"), target_pointer_width = "32"))]
-fn get_lib_dir() -> PathBuf {
-    get_crate_dir().join("mingw/win32/")
+fn get_lib_dir(install_dir: &Path) -> PathBuf {
+    install_dir.join("libsodium-win32/lib/")
 }
 
 #[cfg(all(windows, not(target_env = "msvc"), target_pointer_width = "64"))]
-fn get_lib_dir() -> PathBuf {
-    get_crate_dir().join("mingw/win64/")
-}
-
-fn get_archive(filename: &str) -> std::io::Cursor<Vec<u8>> {
-    use std::fs::File;
-    use std::io::{BufReader, Read};
-
-    let f = File::open(filename).unwrap_or_else(|_| panic!("Failed to open {}", filename));
-    let mut reader = BufReader::new(f);
-    let mut content = Vec::new();
-    reader
-        .read_to_end(&mut content)
-        .unwrap_or_else(|_| panic!("Failed to read {}", filename));
-
-    std::io::Cursor::new(content)
+fn get_lib_dir(install_dir: &Path) -> PathBuf {
+    install_dir.join("libsodium-win64/lib/")
 }
 
 fn get_install_dir() -> PathBuf {
     PathBuf::from(env::var("OUT_DIR").unwrap()).join("installed")
 }
 
-fn build_libsodium() {
-    use libflate::gzip::Decoder;
+fn retrieve_and_verify_archive(filename: &str, signature_filename: &str) -> Vec<u8> {
     use minisign_verify::{PublicKey, Signature};
-    use std::fs::{self, File};
+    use std::fs::File;
     use std::io::prelude::*;
-    use tar::Archive;
 
-    // Determine build target triple
-    let mut target = env::var("TARGET").unwrap();
-    // Hack for RISC-V; Rust apparently uses a different convention for RISC-V triples
-    if target.starts_with("riscv") {
-        let mut split = target.split('-');
-        let arch = split.next().unwrap();
-        let bitness = &arch[5..7];
-        let rest = split.collect::<Vec<_>>().join("-");
-        target = format!("riscv{}-{}", bitness, rest);
-    }
-
-    // Determine filenames
-    let basedir = "libsodium-stable";
-    let basename = "LATEST";
-    let filename = format!("{}.tar.gz", basename);
-    let signature_filename = format!("{}.tar.gz.minisig", basename);
     let pk =
         PublicKey::from_base64("RWQf6LRCGA9i53mlYecO4IzT51TGPpvWucNSCh1CBM0QTaLn73Y7GFO3").unwrap();
 
     let mut archive_bin = vec![];
 
-    #[cfg(feature = "fetch-latest")]
+    #[cfg(any(windows, feature = "fetch-latest"))]
     {
         let baseurl = "https://download.libsodium.org/libsodium/releases";
         let response = ureq::get(&format!("{}/{}", baseurl, filename)).call();
@@ -443,7 +448,7 @@ fn build_libsodium() {
             .unwrap();
     }
 
-    #[cfg(not(feature = "fetch-latest"))]
+    #[cfg(not(any(windows, feature = "fetch-latest")))]
     {
         File::open(&filename)
             .unwrap()
@@ -456,6 +461,34 @@ fn build_libsodium() {
     pk.verify(&archive_bin, &signature, false)
         .expect("Invalid signature");
 
+    archive_bin
+}
+
+fn build_libsodium() {
+    use libflate::gzip::Decoder;
+    use std::fs;
+    use tar::Archive;
+
+    // Determine build target triple
+    let mut target = env::var("TARGET").unwrap();
+    // Hack for RISC-V; Rust apparently uses a different convention for RISC-V triples
+    if target.starts_with("riscv") {
+        let mut split = target.split('-');
+        let arch = split.next().unwrap();
+        let bitness = &arch[5..7];
+        let rest = split.collect::<Vec<_>>().join("-");
+        target = format!("riscv{}-{}", bitness, rest);
+    }
+
+    // Determine filenames
+    let basedir = "libsodium-stable";
+    let basename = "LATEST";
+    let filename = format!("{}.tar.gz", basename);
+    let signature_filename = format!("{}.tar.gz.minisig", basename);
+
+    // Read source archive from disk (or download if requested) & verify signature
+    let archive_bin = retrieve_and_verify_archive(&filename, &signature_filename);
+
     // Determine source and install dir
     let mut install_dir = get_install_dir();
     let mut source_dir = PathBuf::from(env::var("OUT_DIR").unwrap()).join("source");
@@ -478,11 +511,8 @@ fn build_libsodium() {
     fs::create_dir_all(&install_dir).unwrap();
     fs::create_dir_all(&source_dir).unwrap();
 
-    // Get sources
-    let compressed_file = get_archive(&filename);
-
     // Unpack the tarball
-    let gz_decoder = Decoder::new(compressed_file).unwrap();
+    let gz_decoder = Decoder::new(std::io::Cursor::new(archive_bin)).unwrap();
     let mut archive = Archive::new(gz_decoder);
     archive.unpack(&source_dir).unwrap();
     source_dir.push(basedir);
